@@ -31,6 +31,7 @@
 #include "klimitediodevice_p.h"
 
 #include <time.h> // time()
+#include <memory>
 #include "zlib.h"
 
 #ifndef QT_STAT_LNK
@@ -43,8 +44,8 @@
 
 #define BUFFER_SIZE 8*1024
 
-unsigned char k7zip_signature[6] = {'7', 'z', 0xBC, 0xAF, 0x27, 0x1C};
-unsigned char XZ_HEADER_MAGIC[6] = { 0xFD, '7', 'z', 'X', 'Z', 0x00 };
+static const unsigned char k7zip_signature[6] = {'7', 'z', 0xBC, 0xAF, 0x27, 0x1C};
+static const unsigned char XZ_HEADER_MAGIC[6] = { 0xFD, '7', 'z', 'X', 'Z', 0x00 };
 
 #define GetUi16(p, offset) (((unsigned char)p[offset+0]) | (((unsigned char)p[1]) << 8))
 
@@ -160,7 +161,7 @@ static const quint64 k_Bzip2NSIS = 0x040902;
 static const quint64 k_AES = 0x06F10701;
 
 /**
- * A K7ZipFileEntry represents an file in a 7zip archive.
+ * A K7ZipFileEntry represents a file in a 7zip archive.
  */
 class KARCHIVE_EXPORT K7ZipFileEntry : public KArchiveFile
 {
@@ -173,7 +174,7 @@ public:
      * @return the content of this file.
      * Call data() with care (only once per file), this data isn't cached.
      */
-    virtual QByteArray data() const;
+    QByteArray data() const Q_DECL_OVERRIDE;
 
     /**
      * This method returns QIODevice (internal class: KLimitedIODevice)
@@ -185,7 +186,7 @@ public:
      * The returned device auto-opens (in readonly mode), no need to open it.
      * @return the QIODevice of the file
      */
-    virtual QIODevice *createDevice() const;
+    QIODevice *createDevice() const Q_DECL_OVERRIDE;
 
 private:
     const QByteArray m_data;
@@ -224,7 +225,8 @@ public:
         , size(0)
         , crc(0)
         , crcDefined(false)
-    {}
+    {
+    }
 
     QString path;
     bool attribDefined;
@@ -263,7 +265,8 @@ public:
     Folder()
         : unpackCRCDefined(false)
         , unpackCRC(0)
-    {}
+    {
+    }
 
     quint64 getUnpackSize() const
     {
@@ -384,15 +387,15 @@ class K7Zip::K7ZipPrivate
 {
 public:
     K7ZipPrivate(K7Zip *parent)
-        : q(parent),
-          packPos(0),
-          numPackStreams(0),
-          buffer(0),
-          pos(0),
-          end(0),
-          headerSize(0),
-          countSize(0),
-          m_currentFile(0)
+        : q(parent)
+        , packPos(0)
+        , numPackStreams(0)
+        , buffer(0)
+        , pos(0)
+        , end(0)
+        , headerSize(0)
+        , countSize(0)
+        , m_currentFile(0)
     {
     }
 
@@ -506,12 +509,14 @@ public:
 };
 
 K7Zip::K7Zip(const QString &fileName)
-    : KArchive(fileName), d(new K7ZipPrivate(this))
+    : KArchive(fileName)
+    , d(new K7ZipPrivate(this))
 {
 }
 
 K7Zip::K7Zip(QIODevice *dev)
-    : KArchive(dev), d(new K7ZipPrivate(this))
+    : KArchive(dev)
+    , d(new K7ZipPrivate(this))
 {
     Q_ASSERT(dev);
 }
@@ -559,7 +564,7 @@ quint64 K7Zip::K7ZipPrivate::readUInt64()
 
 quint64 K7Zip::K7ZipPrivate::readNumber()
 {
-    if (!buffer) {
+    if (!buffer || (quint64)(pos + 8) > end) {
         return 0;
     }
 
@@ -677,8 +682,8 @@ void K7Zip::K7ZipPrivate::readBoolVector2(int numItems, QVector<bool> &v)
 }
 
 void K7Zip::K7ZipPrivate::readHashDigests(int numItems,
-        QVector<bool> &digestsDefined,
-        QVector<quint32> &digests)
+                                          QVector<bool> &digestsDefined,
+                                          QVector<quint32> &digests)
 {
     if (!buffer) {
         return;
@@ -724,7 +729,7 @@ Folder *K7Zip::K7ZipPrivate::folderItem()
             return 0;
         }
         Folder::FolderInfo *info = new Folder::FolderInfo();
-        unsigned char codecID[codecIdSize];
+        std::unique_ptr<unsigned char[]> codecID(new unsigned char[codecIdSize]);
         for (int i = 0; i < codecIdSize; ++i) {
             codecID[i] = readByte();
         }
@@ -1228,7 +1233,10 @@ public:
     quint32 range;
     quint32 code;
 
-    RangeDecoder() : pos(0) {}
+    RangeDecoder()
+        : pos(0)
+    {
+    }
 
     unsigned char readByte()
     {
@@ -1331,7 +1339,7 @@ public:
 };
 
 template <int numMoveBits>
-class CBitDecoder: public CBitModel<numMoveBits>
+class CBitDecoder : public CBitModel<numMoveBits>
 {
 public:
     quint32 decode(RangeDecoder *decoder)
@@ -1505,15 +1513,15 @@ QByteArray K7Zip::K7ZipPrivate::readAndDecodePackedStreams(bool readMainStreamIn
         QVector<QByteArray> datas;
         for (int j = 0; j < (int)mainCoder->numInStreams; j++) {
             int size = packSizes[j + i];
-            char encodedBuffer[size];
+            std::unique_ptr<char[]> encodedBuffer(new char[size]);
             QIODevice *dev = q->device();
             dev->seek(startPos);
-            quint64 n = dev->read(encodedBuffer, size);
+            quint64 n = dev->read(encodedBuffer.get(), size);
             if (n != (quint64)size) {
                 qDebug() << "Failed read next size, should read " << size << ", read " << n;
                 return inflatedData;
             }
-            QByteArray deflatedData(encodedBuffer, size);
+            QByteArray deflatedData(encodedBuffer.get(), size);
             datas.append(deflatedData);
             startPos += size;
             pos += size;
@@ -1639,7 +1647,7 @@ QByteArray K7Zip::K7ZipPrivate::readAndDecodePackedStreams(bool readMainStreamIn
         }
 
         QByteArray inflated;
-        Q_FOREACH (QByteArray data, inflatedDatas) {
+        Q_FOREACH (const QByteArray& data, inflatedDatas) {
             inflated.append(data);
         }
 
@@ -1661,7 +1669,6 @@ QByteArray K7Zip::K7ZipPrivate::readAndDecodePackedStreams(bool readMainStreamIn
 
 ///////////////// Write ////////////////////
 
-static int sizeItems = 0;
 void K7Zip::K7ZipPrivate::createItemsFromEntities(const KArchiveDirectory *dir, const QString &path, QByteArray &data)
 {
     QStringList l = dir->entries();
@@ -1682,7 +1689,6 @@ void K7Zip::K7ZipPrivate::createItemsFromEntities(const KArchiveDirectory *dir,
             fileInfo->attributes = FILE_ATTRIBUTE_ARCHIVE;
             fileInfo->attributes |= FILE_ATTRIBUTE_UNIX_EXTENSION + ((entry->permissions() & 0xFFFF) << 16);
             fileInfo->size = fileEntry->size();
-            sizeItems += fileInfo->size;
             QString symLink = fileEntry->symLinkTarget();
             if (fileInfo->size > 0) {
                 fileInfo->hasStream = true;
@@ -1869,7 +1875,7 @@ void K7Zip::K7ZipPrivate::writeFolder(const Folder *folder)
 
             quint64 id = info->methodID;
             size_t idSize;
-            for (idSize = 1; idSize < sizeof(id); idSize++) {
+            for (idSize = 1; idSize < sizeof (id); idSize++) {
                 if ((id >> (8 * idSize)) == 0) {
                     break;
                 }
